@@ -214,14 +214,20 @@ class UseCaseReplayer(storytext.guishared.IdleHandlerUseCaseReplayer):
         # so we intercept it and return the right answer for them...
         self.orig_events_pending = gtk.events_pending
         gtk.events_pending = self.events_pending
-
+        self.orig_idle_add = gobject.idle_add
+        gobject.idle_add = self.idle_add
+        self.orig_source_remove = gobject.source_remove
+        gobject.source_remove = self.source_remove
+        self.allIdleHandlers = OrderedDict()
+        
     def addUiMap(self, uiMap):
         self.uiMap = uiMap
         if not self.loggerActive:
             self.tryAddDescribeHandler()
         
     def makeDescribeHandler(self, method):
-        return gobject.idle_add(method, priority=describer.PRIORITY_STORYTEXT_IDLE)
+        if "file chooser to read file system" not in self.waitingForEvents:
+            return gobject.idle_add(method, priority=describer.PRIORITY_STORYTEXT_IDLE)
             
     def tryRemoveDescribeHandler(self):
         if not self.isMonitoring() and not self.readingEnabled: # pragma: no cover - cannot test code with replayer disabled
@@ -230,6 +236,39 @@ class UseCaseReplayer(storytext.guishared.IdleHandlerUseCaseReplayer):
             if self.uiMap:
                 self.uiMap.windows = [] # So we regenerate everything next time around
 
+    def idle_add(self, *args, **kw):
+        handler = self.orig_idle_add(*args, **kw)
+        self.allIdleHandlers[handler] = args, kw, handler
+        return handler
+    
+    def source_remove(self, origHandler):
+        if origHandler in self.allIdleHandlers:
+            actualHandler = self.allIdleHandlers.get(origHandler)[-1]
+            result = self.orig_source_remove(actualHandler)
+            del self.allIdleHandlers[origHandler]
+            return result
+        else:
+            # Added by timeout_add etc
+            return self.orig_source_remove(origHandler)
+
+    def removeAllIdleHandlers(self):
+        idleArgs = []
+        self.logger.debug("Removing idle handlers")
+        for origHandler, data in self.allIdleHandlers.items():
+            handler = data[-1]
+            if handler != describer.idleScheduler.idleHandler:
+                self.removeHandler(handler)
+                idleArgs.append(origHandler)
+        
+        return idleArgs
+
+    def readdAllIdleHandlers(self, idleArgs):
+        self.logger.debug("Readding idle handlers")
+        for origHandler in idleArgs:
+            args, kw, _ = self.allIdleHandlers.get(origHandler)
+            newHandler = self.orig_idle_add(*args, **kw)
+            self.allIdleHandlers[origHandler] = args, kw, newHandler
+            
     def events_pending(self): # pragma: no cover - cannot test code with replayer disabled
         if not self.isActive():
             self.logger.debug("Removing idle handler for descriptions")
@@ -243,8 +282,14 @@ class UseCaseReplayer(storytext.guishared.IdleHandlerUseCaseReplayer):
                 self.tryAddDescribeHandler()
         return return_value
 
+    def events_pending_no_idle_handlers(self): 
+        idleArgs = self.removeAllIdleHandlers()
+        return_value = self.orig_events_pending()
+        self.readdAllIdleHandlers(idleArgs)        
+        return return_value
+    
     def removeHandler(self, handler):
-        gobject.source_remove(handler)
+        return self.orig_source_remove(handler)
 
     def makeTimeoutReplayHandler(self, method, milliseconds):
         return gobject.timeout_add(milliseconds, method, priority=describer.PRIORITY_STORYTEXT_REPLAY_IDLE)
@@ -273,7 +318,7 @@ class UseCaseReplayer(storytext.guishared.IdleHandlerUseCaseReplayer):
         return True # GTK's way of saying the handle should come again
 
     def runMainLoopWithReplay(self):
-        while gtk.events_pending():
+        while self.events_pending_no_idle_handlers():
             gtk.main_iteration()
         if self.delay:
             time.sleep(self.delay)
