@@ -1,26 +1,28 @@
-
+import time, logging
+import storytext.guishared
 
 from storytext.javarcptoolkit import simulator as rcpsimulator
 from storytext.javaswttoolkit import simulator as swtsimulator
 from storytext.javaswttoolkit.util import getInt
 from storytext.definitions import UseCaseScriptError
-import org.eclipse.swtbot.eclipse.gef.finder as gefbot
-from org.eclipse.swtbot.swt.finder.exceptions import WidgetNotFoundException
-from org.eclipse.jface.viewers import ISelectionChangedListener
-from org.eclipse.ui.internal import EditorReference
-# Force classloading in the test thread where it works...
-from org.eclipse.draw2d.geometry import Insets #@UnusedWildImport
-from org.eclipse.draw2d import * #@UnusedWildImport
-from org.eclipse.gef import *
-import storytext.guishared
-from org.eclipse import swt
-import time, logging
 
-class StoryTextSWTBotGefViewer(gefbot.widgets.SWTBotGefViewer):
+from org.eclipse.draw2d import FigureCanvas
+from org.eclipse.draw2d.geometry import Insets
+from org.eclipse.gef import GraphicalViewer
+from org.eclipse.jface.viewers import ISelectionChangedListener
+from org.eclipse.swt import SWT
+from org.eclipse.swt.events import DragDetectListener, MouseAdapter
+from org.eclipse.ui.internal import EditorReference
+
+from org.eclipse.swtbot.eclipse.gef.finder import SWTGefBot
+from org.eclipse.swtbot.eclipse.gef.finder.widgets import SWTBotGefFigureCanvas, SWTBotGefEditPart, SWTBotGefViewer
+from org.eclipse.swtbot.swt.finder.exceptions import WidgetNotFoundException
+
+class StoryTextSWTBotGefViewer(SWTBotGefViewer):
     widgetMonitor = None
     def __init__(self, botOrGefViewer):
-        gefViewer = self._getViewer(botOrGefViewer) if isinstance(botOrGefViewer, gefbot.widgets.SWTBotGefViewer) else botOrGefViewer
-        gefbot.widgets.SWTBotGefViewer.__init__(self, gefViewer)
+        gefViewer = self._getViewer(botOrGefViewer) if isinstance(botOrGefViewer, SWTBotGefViewer) else botOrGefViewer
+        SWTBotGefViewer.__init__(self, gefViewer)
         self.logger = logging.getLogger("Centre finding")
 
     def findOverlap(self, overlaps, centre):
@@ -132,7 +134,7 @@ class StoryTextSWTBotGefViewer(gefbot.widgets.SWTBotGefViewer):
 
     def getBoundsInternal(self, editPart):
         # getAbsoluteBounds method has private access modifier
-        declaredMethod = self.getClass().getSuperclass().getDeclaredMethod("getAbsoluteBounds", [gefbot.widgets.SWTBotGefEditPart])
+        declaredMethod = self.getClass().getSuperclass().getDeclaredMethod("getAbsoluteBounds", [SWTBotGefEditPart])
         declaredMethod.setAccessible(True)
         return declaredMethod.invoke(self, [editPart])
         
@@ -141,7 +143,7 @@ class StoryTextSWTBotGefViewer(gefbot.widgets.SWTBotGefViewer):
     
     @staticmethod
     def _getViewer(obj):
-        viewerField = gefbot.widgets.SWTBotGefViewer.getDeclaredField("graphicalViewer")
+        viewerField = SWTBotGefViewer.getDeclaredField("graphicalViewer")
         viewerField.setAccessible(True)
         return viewerField.get(obj)
     
@@ -154,44 +156,49 @@ class StoryTextSWTBotGefViewer(gefbot.widgets.SWTBotGefViewer):
         viewerField.setAccessible(True)
         viewerField.set(self, figureCanvas)
 
-    # toX and toY should be in display coordinates
+    # toX and toY should be in display coordinates. Widget coordinates don't work as we may be dragging from one widget to another
     def drag(self, editPart, toX, toY, keyModifiers=0):
-        centreX, centreY = self.getCenter(editPart)
+        widgetCentreX, widgetCentreY = self.getCenter(editPart)
+        displayCentre = swtsimulator.runOnUIThread(self.getFigureCanvas().toDisplayLocation, widgetCentreX, widgetCentreY)
+        centreX = displayCentre.x
+        centreY = displayCentre.y
+        self.logger.debug("Dragging " + repr((centreX, centreY)) + " to " + repr((toX, toY)))
         self.getFigureCanvas().mouseDrag(centreX, centreY, toX, toY, keyModifiers)
  
 
-class StoryTextSWTBotGefFigureCanvas(gefbot.widgets.SWTBotGefFigureCanvas):
+class StoryTextSWTBotGefFigureCanvas(SWTBotGefFigureCanvas):
     def __init__(self, *args, **kw):
-        gefbot.widgets.SWTBotGefFigureCanvas.__init__(self, *args, **kw)
+        SWTBotGefFigureCanvas.__init__(self, *args, **kw)
         self.eventPoster = swtsimulator.EventPoster(self.display)
         
     def mouseDrag(self, fromX, fromY, toX, toY, keyModifiers=0):
-        self._mouseDrag(fromX, fromY, toX, toY, keyModifiers)
-
-    def _mouseDrag(self, fromX, fromY, toX, toY, keyModifiers=0):
-        fromConverted = swtsimulator.runOnUIThread(self.toDisplayLocation, fromX, fromY)
-        fromX = fromConverted.x
-        fromY = fromConverted.y
+        # These values, and the broad algorithm here, are taken from a draft implementation of DnD for SWTBot
+        # The code is attached to the bug at https://bugs.eclipse.org/bugs/show_bug.cgi?id=285271
+        # When it is integrated into SWTBot we should probably use it directly
+        dragThreshold = 16
+        dragDelay = 0.3
         self.eventPoster.moveMouseAndWait(fromX, fromY)
                
-        counterX, counterY = self.getCounters(fromX, toX, fromY, toY)
         self.eventPoster.checkAndPostKeyPressed(keyModifiers)
-        swtsimulator.runOnUIThread(storytext.guishared.catchAll, self.startDrag, fromX, toX, fromY, toY, counterX*10, counterY*10)
-        self.moveDragged(fromX, toX, fromY, toY)
+        initialDragTargetX = fromX + dragThreshold
+        initialDragTargetY = fromY
+        swtsimulator.runOnUIThread(storytext.guishared.catchAll, self.startDrag, initialDragTargetX, initialDragTargetY)
+        time.sleep(dragDelay)
+        self.moveDragged(initialDragTargetX, initialDragTargetY, toX, toY)
         swtsimulator.runOnUIThread(storytext.guishared.catchAll, self.eventPoster.postMouseUp)
         self.eventPoster.checkAndPostKeyReleased(keyModifiers)
 
-    def startDrag(self, fromX, toX, fromY, toY, offsetX, offsetY):
+    def startDrag(self, targetX, targetY):
         self.eventPoster.postMouseDown()
-        self.eventPoster.postMouseMove(fromX + offsetX, fromY + offsetY)
+        self.eventPoster.postMouseMove(targetX, targetY)
        
-    def getCounters(self, x1, x2, y1, y2):
+    def getCounters(self, x1, y1, x2, y2):
         counterX = 1 if x1 < x2 else -1
         counterY = 1 if y1 < y2 else -1
         return counterX, counterY
     
-    def moveDragged(self, fromX, toX, fromY, toY):
-        counterX, counterY = self.getCounters(fromX, toX, fromY, toY)
+    def moveDragged(self, fromX, fromY, toX, toY):
+        counterX, counterY = self.getCounters(fromX, fromY, toX, toY)
         startX = fromX
         startY = fromY
         while startX != toX:
@@ -243,7 +250,7 @@ class WidgetMonitor(rcpsimulator.WidgetMonitor):
         StoryTextSWTBotGefViewer.widgetMonitor = self
 
     def createSwtBot(self):
-        return gefbot.SWTGefBot()
+        return SWTGefBot()
 
     def monitorAllWidgets(self, widgets):
         rcpsimulator.WidgetMonitor.monitorAllWidgets(self, widgets)
@@ -430,6 +437,8 @@ class ViewerEvent(storytext.guishared.GuiEvent):
         return "of type " + self.widget.getType()
 
 class ViewerSelectEvent(ViewerEvent):
+    # Allow customwidgetsevents to set this
+    pauseBetweenSelections = 0
     def connectRecord(self, method):
         class SelectionListener(ISelectionChangedListener):
             def selectionChanged(lself, event): #@NoSelf
@@ -467,22 +476,18 @@ class ViewerSelectEvent(ViewerEvent):
         if len(parts) == 1 and not partial:
             self.widget.clickOnCenter(parts[0])
         else:
-            pauseTime = self.getPauseBetweenSelections()
             for i, part in enumerate(parts):
-                if i > 0 and pauseTime:
-                    time.sleep(pauseTime)
-                self.widget.clickOnCenter(part, swt.SWT.CTRL)
+                if i > 0 and self.pauseBetweenSelections:
+                    time.sleep(self.pauseBetweenSelections)
+                self.widget.clickOnCenter(part, SWT.CTRL)
                 
-    def getPauseBetweenSelections(self):
-        return 0 #Don't pause by default, allow customwidgetevents to override this
-
     @classmethod
     def getAssociatedSignal(cls, widget):
         return "Select"
     
     @classmethod
     def getSignalsToFilter(cls):
-        return [ swt.SWT.MouseDown, swt.SWT.MouseUp ]
+        return [ SWT.MouseDown, SWT.MouseUp ]
 
     def shouldRecord(self, parts, *args):
         types = self.getSignalsToFilter()
@@ -497,19 +502,19 @@ class ViewerSelectEvent(ViewerEvent):
         return True
 
 class DragHolder():
-    draggedPart = None
+    draggedParts = []
     sourceEvent = None
     sourceSwtEvent = None
     @classmethod
     def reset(cls):
-        cls.draggedPart = None
+        cls.draggedParts = []
         cls.sourceEvent = None
         cls.sourceSwtEvent = None
     
 class ViewerDragAndDropEvent(ViewerEvent):
     allInstances = {}
     def connectRecord(self, method):
-        class DDListener(swt.events.DragDetectListener):
+        class DDListener(DragDetectListener):
             def dragDetected(lself, event):#@NoSelf
                 storytext.guishared.catchAll(self.applyToDragged, event, method)
 
@@ -518,12 +523,12 @@ class ViewerDragAndDropEvent(ViewerEvent):
 
     def applyToDragged(self, event, method):
         if len(self.widget.selectedEditParts()) > 0 and self.shouldDrag(event):
-            DragHolder.draggedPart = self.widget.selectedEditParts()[0].part()#self.getGefViewer().findObjectAt(p)
+            DragHolder.draggedParts = [ e.part() for e in self.widget.selectedEditParts() ]#self.getGefViewer().findObjectAt(p)
             DragHolder.sourceEvent = self
             DragHolder.sourceSwtEvent = event
 
     def addDropListener(self, method):
-        class MListener(swt.events.MouseAdapter):
+        class MListener(MouseAdapter):
             def mouseUp(lself, event):#@NoSelf
                 storytext.guishared.catchAll(self.handleDrop, event, method)
 
@@ -532,26 +537,32 @@ class ViewerDragAndDropEvent(ViewerEvent):
 
     def handleDrop(self, event, method):
         if DragHolder.sourceEvent is not None and self.__class__ is DragHolder.sourceEvent.__class__:
-            for instance in self.allInstances.get(self.__class__, []):
-            # No guarantee MouseUp appers on same widget
-                dropX, dropY = instance.getDropPosition(event)
-                if dropX is not None and instance.getControl().isVisible() and instance.isValidDragAndDrop(DragHolder.sourceSwtEvent, event, dropX, dropY):
-                    method(DragHolder.draggedPart, dropX, dropY, instance, DragHolder.sourceEvent)
-                    break
-            DragHolder.reset()
+            try:
+                for instance in self.allInstances.get(self.__class__, []):
+                    # No guarantee MouseUp appers on same widget
+                    dropX, dropY = instance.getDropPosition(event)
+                    if instance.isValidDragAndDrop(instance.getControl(), dropX, dropY):
+                        method(DragHolder.draggedParts, dropX, dropY, instance, DragHolder.sourceEvent)
+                        break
+            finally:
+                DragHolder.reset()
 
-    def isValidDragAndDrop(self, dragEvent, dropEvent, dropX, dropY):
-        return dragEvent.widget is not dropEvent.widget or dragEvent.x != dropX or dragEvent.y != dropY
+    def isValidDragAndDrop(self, dropWidget, dropX, dropY):
+        if dropX is None or not dropWidget.isVisible():
+            return False
+        
+        dragEvent = DragHolder.sourceSwtEvent
+        return dragEvent.widget is not dropWidget or dragEvent.x != dropX or dragEvent.y != dropY
 
     def getDropPosition(self, event):
         return event.x, event.y
 
-    def getStateDescription(self, part, *args):
-        partDesc = self.storeObjectDescription(part)
-        targetDesc = self.getTargetDescription(part, *args)
+    def getStateDescription(self, parts, *args):
+        partDesc = ",".join(map(self.storeObjectDescription, parts))
+        targetDesc = self.getTargetDescription(parts, *args)
         return partDesc + " to " + targetDesc
 
-    def getTargetDescription(self, part, x, y, *args):
+    def getTargetDescription(self, parts, x, y, *args):
         return str(x) + ":" + str(y)
 
     @classmethod
@@ -560,23 +571,36 @@ class ViewerDragAndDropEvent(ViewerEvent):
     
     @classmethod
     def getSignalsToFilter(cls):
-        return [ swt.SWT.MouseUp ]
+        return [ SWT.MouseUp ]
     
-    def shouldRecord(self, part, *args):
+    def shouldRecord(self, parts, x, y, *events):
+        controls = []
+        for event in events:
+            if hasattr(event, "getControl"):
+                controls.append(event.getControl())
         types = self.getSignalsToFilter()
-        return DisplayFilter.instance.hasEventOfType(types, self.getControl())
+        return any((DisplayFilter.instance.hasEventOfType(types, ctrl) for ctrl in controls))
 
-    def generate(self, partAndPos):
-        self.widget.drag(*partAndPos)
+    def generate(self, args, **kw):
+        parts, x, y = args
+        if len(parts) > 1:
+            for part in parts[:-1]:
+                self.widget.clickOnCenter(part, SWT.CTRL)
+                if ViewerSelectEvent.pauseBetweenSelections:
+                    time.sleep(ViewerSelectEvent.pauseBetweenSelections)
+        self.widget.drag(parts[-1], x, y, **kw)
         
     def parseArguments(self, description):
         sourceDesc, dest = description.split(" to ", 1)
-        editPart = self.findEditPart(self.widget.rootEditPart(), sourceDesc)
-        if not editPart:
-            raise UseCaseScriptError, "could not find any objects in viewer matching description '" + sourceDesc + "'"
-        
+        allParts = sourceDesc.split(",")
+        editParts = []
+        for partDesc in allParts:
+            editPart = self.findEditPart(self.widget.rootEditPart(), partDesc)
+            if not editPart:
+                raise UseCaseScriptError, "could not find any objects in viewer matching description '" + sourceDesc + "'"
+            editParts.append(editPart)
         displayLocation = self.parseDestination(dest)
-        return editPart, displayLocation.x, displayLocation.y
+        return editParts, displayLocation.x, displayLocation.y
     
     def parseDestination(self, dest):
         if ":" not in dest:
