@@ -10,12 +10,12 @@ class ScriptEngine(javaswttoolkit.ScriptEngine):
         return UseCaseReplayer(self.uiMap, universalLogging, self.recorder, **kw)
 
     def runSystemUnderTest(self, args):
-        import org.eclipse.equinox.launcher as launcher
+        from org.eclipse.equinox.launcher import Main
         cmdArgs = [ "-application", self.testscriptPlugin + ".application",
                     "-testApplication" ] + args
         log = logging.getLogger("gui log")
         log.debug("Starting application with args : " + " ".join(cmdArgs))
-        launcher.Main.main(cmdArgs)
+        Main.main(cmdArgs)
         
 
     def getDefaultTestscriptPluginName(self):
@@ -43,7 +43,7 @@ class TestRunner(Runnable):
         # to use the same one, or things won't work
         sys.classLoader = Thread.currentThread().getContextClassLoader()
         self.method()
-
+    
         
 class UseCaseReplayer(javaswttoolkit.UseCaseReplayer):
     def __init__(self, *args, **kw):
@@ -61,49 +61,19 @@ class UseCaseReplayer(javaswttoolkit.UseCaseReplayer):
             sys.stderr.write("ERROR: Could not find SWTBot testscript plugin. Please install it as described at :\n" +
                              "http://www.texttest.org/index.php?page=ui_testing&n=storytext_and_swt\n")
             sys.exit(1)
-
+        
+    def initEclipsePackagesWithDisplay(self):
+        # Just necessary for documentation mechanism to work( Getting a weird gtk error otherwise)
+        from org.eclipse.swt.dnd import Clipboard, TextTransfer
+    
     def runOnRecordExit(self): # pragma: no cover - cannot test with replayer disabled
         if not self.disable_usecase_names:
             self.uiMap.scriptEngine.replaceAutoRecordingForUsecase("javaswt", exitHook=True)
-        self.tryTerminateCoverage()
+        
+        # Eclipse doesn't return control to the python interpreter
+        # So we terminate coverage manually at this point if we're measuring it
+        self.uiMap.scriptEngine.tryTerminateCoverage()
 
     def enableJobListener(self):
         from jobsynchroniser import JobListener
         JobListener.enable()
-
-    def tryTerminateCoverage(self):
-        # Eclipse doesn't return control to the python interpreter
-        # So we terminate coverage manually at this point if we're measuring it
-        try:
-            import coverage #@UnresolvedImport
-            coverage.process_shutdown()
-        except: # pragma: no cover - Obviously can't measure coverage here!
-            pass
-        
-    def shouldReraise(self, e, clsName):
-        msg = str(e).strip()
-        allowedMessages = [ "No module named customwidgetevents",
-                            "cannot import name " + clsName ]
-        return msg not in allowedMessages
-
-    def getDescriberClass(self):
-        try:
-            from customwidgetevents import Describer
-        except ImportError, e:
-            if self.shouldReraise(e, "Describer"):
-                raise
-            try:
-                from draw2ddescriber import Describer
-                return Describer
-            except ImportError:
-                from describer import Describer
-        return Describer
-    
-    def getMonitorClass(self):
-        try:
-            from customwidgetevents import WidgetMonitor
-        except ImportError, e:
-            if self.shouldReraise(e, "WidgetMonitor"):
-                raise
-            from simulator import WidgetMonitor
-        return WidgetMonitor
