@@ -5,25 +5,22 @@ from storytext.javaswttoolkit import simulator as swtsimulator
 from storytext.javaswttoolkit import util
 import storytext.guishared
 from storytext.definitions import UseCaseScriptError
-import org.eclipse.swtbot.eclipse.finder as swtbot
-from org.eclipse.ui import PlatformUI, IPartListener, IPropertyListener, IWorkbenchPartConstants
 
-# If classes get mentioned for the first time in the event dispatch thread, they will get the wrong classloader
-# So we load everything we'll ever need here, once, where we know what the classloader is
-from org.eclipse.swt.widgets import * #@UnusedWildImport
-from org.eclipse.swt.custom import * #@UnusedWildImport
-from org.eclipse.swt.dnd import * #@UnusedWildImport
-from org.eclipse.swt.layout import * #@UnusedWildImport
-from org.eclipse.swtbot.swt.finder.finders import * #@UnusedWildImport
-from org.eclipse.ui.dialogs import FilteredTree
-from org.eclipse.swt.widgets import Event
+from java.lang import NullPointerException, Integer, IllegalArgumentException
+
+from org.eclipse.jface.bindings.keys import KeyStroke 
 from org.eclipse.swt import SWT
+from org.eclipse.swt.custom import CTabItem
+from org.eclipse.swt.widgets import Event, Listener, MenuItem, Text
+from org.eclipse.ui import PlatformUI, IPartListener, IPropertyListener, IWorkbenchPartConstants
+from org.eclipse.ui.dialogs import FilteredTree
 from org.eclipse.ui.forms.widgets import ExpandableComposite
 from org.eclipse.ui.forms.events import ExpansionAdapter
-from org.eclipse.swtbot.swt.finder.widgets import AbstractSWTBotControl
-from org.eclipse.jface.bindings.keys import KeyStroke 
-from java.lang import NullPointerException, Integer
 
+from org.eclipse.swtbot.swt.finder.widgets import AbstractSWTBotControl
+from org.eclipse.swtbot.eclipse.finder import SWTWorkbenchBot
+from org.eclipse.swtbot.eclipse.finder.widgets import SWTBotView
+ 
 class WidgetAdapter(swtsimulator.WidgetAdapter):
     widgetViewIds = {}    
     swtsimulator.WidgetAdapter.secondaryIdentifiers.append("View")
@@ -100,7 +97,7 @@ class WidgetMonitor(swtsimulator.WidgetMonitor):
         swtsimulator.WidgetMonitor.__init__(self, *args, **kw)
             
     def createSwtBot(self):
-        return swtbot.SWTWorkbenchBot()
+        return SWTWorkbenchBot()
     
     def getDisplayFilterClass(self):
         return DisplayFilter
@@ -227,8 +224,11 @@ class RCPSelectEvent(swtsimulator.SelectEvent):
         # Aim is to handle Eclipse RCP key binding mechanism, not SWT accelerators which work anyway
         if "\t" in text and widget.getAccelerator() == 0:
             keyBinding = text.split("\t")[-1].lower()
-            keyStroke = KeyStroke.getInstance(keyBinding)
-            self.bindingListener.mapKeyBinding(keyStroke, method, widget.getDisplay(), self)
+            try:
+                keyStroke = KeyStroke.getInstance(keyBinding)
+                self.bindingListener.mapKeyBinding(keyStroke, method, widget.getDisplay(), self)
+            except IllegalArgumentException:
+                pass # Tab characters don't have to imply an accelerator, a menu item label can contain these anyway
             
     def shouldRecord(self, event, *args):
         if event.type == SWT.KeyDown:
@@ -263,10 +263,15 @@ class PartActivateEvent(swtsimulator.SignalEvent):
         # The idea is to just do this, but it seems to cause strange things to happen
         #internally. So we do it outside SWTBot instead.
         #self.widget.setFocus()
-        swtsimulator.runOnUIThread(self.clickMatchingTab, *args)
-        #page = self.widget.getViewReference().getPage()
-        #view = self.widget.getViewReference().getView(False)
-        #swtsimulator.runOnUIThread(page.activate, view)
+        if self.actuallyClick():
+            swtsimulator.runOnUIThread(self.clickMatchingTab, *args)
+        else:
+            page = self.widget.getViewReference().getPage()
+            view = self.widget.getViewReference().getView(False)
+            swtsimulator.runOnUIThread(page.activate, view)
+        
+    def actuallyClick(self):
+        return "clicked" in self.generationModifiers
         
     def getTabFolder(self):
         for tab in self.widget.getViewReference().getPane().getTabList():
@@ -299,17 +304,19 @@ class PartActivateEvent(swtsimulator.SignalEvent):
         else:
             raise UseCaseScriptError, "Could not find View named '" + argumentString + "' to activate.\n" + \
                 "Views are named " + repr([ str(i.getTitleArgument()) for i in self.allInstances ])
-            
 
     def getTitle(self):
         return self.widget.getViewReference().getPartName()
 
     def shouldRecord(self, part, *args):
-        # TODO: Need to check no other events are waiting in DisplayFilter 
-        return self.hasMultipleViews() and \
-            (not swtsimulator.DisplayFilter.instance.hasEvents() or \
-            swtsimulator.DisplayFilter.instance.hasEventOfType(self.getSignalsToFilter(), self.getTabFolder()))
-    
+        if not self.hasMultipleViews():
+            return False
+        
+        if self.actuallyClick():
+            return swtsimulator.DisplayFilter.instance.hasEventOfType(self.getSignalsToFilter(), self.getTabFolder())
+        else:
+            return not swtsimulator.DisplayFilter.instance.hasEvents()
+        
     def hasMultipleViews(self):
         # If there is only one view, don't try to record if it's activated, it's probably just been created...
         return sum((i.isActivatable() for i in self.allInstances)) > 1
@@ -469,7 +476,7 @@ class ExpandableCompositeEvent(swtsimulator.SelectEvent):
 
         swtsimulator.runOnUIThread(self.widget.widget.widget.addExpansionListener, RecordListener())
             
-swtsimulator.eventTypes.append((swtbot.widgets.SWTBotView, [ PartActivateEvent, PartCloseEvent ]))
+swtsimulator.eventTypes.append((SWTBotView, [ PartActivateEvent, PartCloseEvent ]))
 swtsimulator.eventTypes.append((SWTBotExpandableComposite, [ ExpandableCompositeEvent ]))
 
 replacements = [(swtsimulator.SelectEvent, RCPSelectEvent),
