@@ -2,37 +2,116 @@
 import storytext.guishared, util, types, logging, sys, os
 from storytext.definitions import UseCaseScriptError
 from storytext.gridformatter import GridFormatter
-from org.eclipse import swt
+
+
 from browserhtmlparser import BrowserHtmlParser
 from java.util import Date
 from java.io import File, FilenameFilter
-from org.eclipse.jface.resource import ImageDescriptor
+
 from array import array
 from ordereddict import OrderedDict
 
+from org.eclipse.jface.resource import ImageDescriptor
+
+from org.eclipse.swt import SWT
+from org.eclipse.swt.browser import Browser
+from org.eclipse.swt.custom import CCombo, CLabel, CTabFolder, StackLayout
+#from org.eclipse.swt.dnd import Clipboard, TextTransfer
+from org.eclipse.swt.graphics import Color, GC, Image, ImageLoader
+from org.eclipse.swt.layout import GridLayout, FillLayout, FormLayout, RowLayout
+from org.eclipse.swt.widgets import Button, Canvas, Combo, Composite, Control, CoolBar, CoolItem, DateTime, \
+    Dialog, Display, Event, ExpandBar, Group, Item, Label, Link, List, Listener, Menu, MenuItem, Sash, Shell, \
+    Spinner, TabFolder, Table, Text, ToolBar, Tree
+        
+class ColorNameFinder:
+    def __init__(self):
+        self.names = {}
+        self.widgetDefaults = set()
+        # Add java.awt colors  
+        from java.awt import Color as awtColor
+        self.addColors(awtColor, postfix="&")
+        
+    def shortenColorName(self, name, abbreviations):
+        ret = name.lower()
+        for text, repl in abbreviations:
+            ret = ret.replace(text, repl)
+        return ret
+
+    def addColor(self, name, color, postfix="", modifiers=[], abbreviations=[]):
+        if hasattr(color, "getRed"):
+            newName = name + postfix
+            nameToUse = self.shortenColorName(newName, abbreviations)
+            self.names[self.getRGB(color)] = nameToUse
+            for modifier, prefix in modifiers:
+                rgb = self.getRGB(self.applyModifier(modifier, color))
+                if rgb not in self.names:
+                    self.names[rgb] = prefix + nameToUse
+            
+    def applyModifier(self, modifier, color):
+        try:
+            return modifier(color)
+        except:
+            colorToUse = Color(Display.getDefault(), color.getRed(), color.getGreen(), color.getBlue())
+            return modifier(colorToUse)
+
+    def addColors(self, cls, **kw):
+        for name in sorted(cls.__dict__):
+            if not name.startswith("__"):
+                try:
+                    color = getattr(cls, name)
+                    self.addColor(name, color, **kw)
+                except AttributeError:
+                    pass
+                    
+    def addSWTColors(self, display):
+        for name in sorted(SWT.__dict__):
+            if name.startswith("COLOR_"):
+                colorKey = getattr(SWT, name)
+                color = display.getSystemColor(colorKey)
+                rgb = self.getRGB(color)
+                # Have to do this last because we can only retrieve them in the UI thread
+                # Don't override any custom names we might have
+                if rgb not in self.names:
+                    self.names[rgb] = name[6:].lower()
+                if "WIDGET" in name:
+                    self.widgetDefaults.add(rgb)
+                
+    def getRGB(self, color):
+        return color.getRed(), color.getGreen(), color.getBlue()
+
+    def getName(self, color):
+        return self.names.get(self.getRGB(color), "unknown")
+    
+    def getNameForWidget(self, color):
+        rgb = self.getRGB(color)
+        return self.names.get(rgb, "unknown") if rgb not in self.widgetDefaults else ""
+        
+        
+colorNameFinder = ColorNameFinder()
+
 
 class Describer(storytext.guishared.Describer):
-    styleNames = [ (swt.widgets.CoolItem, []),
-                   (swt.widgets.Item    , [ "SEPARATOR", "DROP_DOWN", "CHECK", "CASCADE", "RADIO" ]),
-                   (swt.widgets.Button  , [ "CHECK", "RADIO", "TOGGLE", "ARROW", "UP", "DOWN" ]),
-                   (swt.widgets.DateTime, [ "DATE", "TIME", "CALENDAR", "SHORT" ]),
-                   (swt.widgets.Combo   , [ "READ_ONLY", "SIMPLE" ]),
-                   (swt.custom.CCombo   , [ "READ_ONLY", "FLAT", "BORDER" ]), 
-                   (swt.widgets.Text    , [ "PASSWORD", "SEARCH", "READ_ONLY" ]) ]
+    styleNames = [ (CoolItem, []),
+                   (Item    , [ "SEPARATOR", "DROP_DOWN", "CHECK", "CASCADE", "RADIO" ]),
+                   (Button  , [ "CHECK", "RADIO", "TOGGLE", "ARROW", "UP", "DOWN" ]),
+                   (DateTime, [ "DATE", "TIME", "CALENDAR", "SHORT" ]),
+                   (Combo   , [ "READ_ONLY", "SIMPLE" ]),
+                   (CCombo   , [ "READ_ONLY", "FLAT", "BORDER" ]), 
+                   (Text    , [ "PASSWORD", "SEARCH", "READ_ONLY" ]) ]
     ignoreWidgets = [ types.NoneType ]
     # DateTime children are an implementation detail
     # Coolbars, Toolbars and Expandbars describe their children directly : they have two parallel children structures
-    ignoreChildren = (swt.widgets.CoolBar, swt.widgets.ExpandBar, swt.widgets.ToolBar, swt.widgets.DateTime, swt.widgets.Group)
-    statelessWidgets = [ swt.widgets.Sash ]
-    stateWidgets = [ swt.widgets.Shell, swt.widgets.Button, swt.widgets.Menu, swt.widgets.Link,
-                     swt.widgets.CoolBar, swt.widgets.ToolBar, swt.widgets.Label, swt.custom.CLabel,
-                     swt.widgets.Combo, swt.widgets.ExpandBar, swt.widgets.Text, swt.widgets.List,
-                     swt.widgets.Tree, swt.widgets.DateTime, swt.widgets.TabFolder, swt.widgets.Table, 
-                     swt.custom.CTabFolder, swt.widgets.Canvas, swt.browser.Browser, swt.custom.CCombo,
-                     swt.widgets.Spinner, swt.widgets.Group, swt.widgets.Composite ]
+    ignoreChildren = (CoolBar, ExpandBar, ToolBar, DateTime, Group)
+    statelessWidgets = [ Sash ]
+    stateWidgets = [ Shell, Button, Menu, Link,
+                     CoolBar, ToolBar, Label, CLabel,
+                     Combo, ExpandBar, Text, List,
+                     Tree, DateTime, TabFolder, Table, 
+                     CTabFolder, Canvas, Browser, CCombo,
+                     Spinner, Group, Composite ]
     childrenMethodName = "getChildren"
     visibleMethodName = "getVisible"
-    def __init__(self):
+    def __init__(self, canvasDescriberClasses=[]):
         storytext.guishared.Describer.__init__(self)
         self.canvasCounter = storytext.guishared.WidgetCounter()
         self.contextMenuCounter = storytext.guishared.WidgetCounter(self.contextMenusEqual)
@@ -47,6 +126,8 @@ class Describer(storytext.guishared.Describer):
         self.imageToName = {}
         self.handleImages()
         self.screenshotNumber = 0
+        self.colorsAdded = False
+        self.canvasDescriberClasses = canvasDescriberClasses
 
     def setWidgetPainted(self, widget):
         if widget not in self.widgetsDescribed and widget not in self.windows and widget not in self.widgetsAppeared:
@@ -56,56 +137,56 @@ class Describer(storytext.guishared.Describer):
     def setWidgetShown(self, widget):
         # Menu show events seem a bit spurious, they aren't really shown at this point:
         # ScrollBar shows are not relevant to anything
-        if isinstance(widget, swt.widgets.Control) and widget not in self.widgetsAppeared:
+        if isinstance(widget, Control) and widget not in self.widgetsAppeared:
             self.logger.debug("Widget shown " + self.getRawData(widget))
             self.widgetsAppeared.append(widget)
             if widget in self.widgetsMoved:
                 self.widgetsMoved.remove(widget)
             
     def setWidgetMoved(self, widget):
-        if isinstance(widget, swt.widgets.Control) and widget not in self.widgetsAppeared and widget.getParent() not in self.parentsResized:
+        if isinstance(widget, Control) and widget not in self.widgetsAppeared and widget.getParent() not in self.parentsResized:
             self.logger.debug("Widget moved " + self.getRawData(widget))
             self.widgetsMoved.append(widget)
         
     def setWidgetResized(self, widget):
-        if isinstance(widget, swt.widgets.Control):
+        if isinstance(widget, Control):
             self.parentsResized.add(widget)
             self.parentsResized.add(widget.getParent())
                     
     def addFilters(self, display):
-        class ShowListener(swt.widgets.Listener):
-            def handleEvent(listenerSelf, e):
+        class ShowListener(Listener):
+            def handleEvent(listenerSelf, e): #@NoSelf
                 storytext.guishared.catchAll(self.setWidgetShown, e.widget)
 
-        class PaintListener(swt.widgets.Listener):
-            def handleEvent(listenerSelf, e):
+        class PaintListener(Listener):
+            def handleEvent(listenerSelf, e): #@NoSelf
                 storytext.guishared.catchAll(self.setWidgetPainted, e.widget)
 
-        class MoveListener(swt.widgets.Listener):
-            def handleEvent(listenerSelf, e):
+        class MoveListener(Listener):
+            def handleEvent(listenerSelf, e): #@NoSelf
                 storytext.guishared.catchAll(self.setWidgetMoved, e.widget)
 
-        class ResizeListener(swt.widgets.Listener):
-            def handleEvent(listenerSelf, e):
+        class ResizeListener(Listener):
+            def handleEvent(listenerSelf, e): #@NoSelf
                 storytext.guishared.catchAll(self.setWidgetResized, e.widget)
 
-        display.addFilter(swt.SWT.Show, ShowListener())
-        display.addFilter(swt.SWT.Paint, PaintListener())
-        display.addFilter(swt.SWT.Move, MoveListener())
-        display.addFilter(swt.SWT.Resize, ResizeListener())
-        display.addFilter(swt.SWT.Dispose, ResizeListener()) # Being disposed is the ultimate resize :)
+        display.addFilter(SWT.Show, ShowListener())
+        display.addFilter(SWT.Paint, PaintListener())
+        display.addFilter(SWT.Move, MoveListener())
+        display.addFilter(SWT.Resize, ResizeListener())
+        display.addFilter(SWT.Dispose, ResizeListener()) # Being disposed is the ultimate resize :)
 
     def getScreenshotFileName(self, screenshotDir):
         return os.path.join(screenshotDir, "screenshot" + str(self.screenshotNumber) + ".png")
 
     def writeScreenshot(self, shell):
         display = shell.getDisplay()
-        gc = swt.graphics.GC(display);
-        image = swt.graphics.Image(display, shell.getBounds())
+        gc = GC(display);
+        image = Image(display, shell.getBounds())
         gc.copyArea(image, shell.getBounds().x, shell.getBounds().y)
         gc.dispose()
         
-        imageLoader = swt.graphics.ImageLoader()
+        imageLoader = ImageLoader()
         imageLoader.data = [ image.getImageData() ]
         self.screenshotNumber += 1
         screenshotDir = os.path.join(os.getenv("TEXTTEST_LOG_DIR", os.getcwd()), "screenshots")
@@ -115,12 +196,16 @@ class Describer(storytext.guishared.Describer):
         while os.path.isfile(fileName):
             self.screenshotNumber += 1
             fileName = self.getScreenshotFileName(screenshotDir)
-        imageLoader.save(fileName, swt.SWT.IMAGE_PNG) 
+        imageLoader.save(fileName, SWT.IMAGE_PNG) 
     
     def describeWithUpdates(self, shellMethod):
         shell = shellMethod()
         if self.writeScreenshots:
             self.writeScreenshot(shell)
+        if not self.colorsAdded:
+            self.colorsAdded = True
+            colorNameFinder.addSWTColors(shell.getDisplay())
+
         if shell in self.windows:
             stateChanges = self.findStateChanges(shell)
             stateChangeWidgets = [ widget for widget, _, _ in stateChanges ]
@@ -131,9 +216,8 @@ class Describer(storytext.guishared.Describer):
             self.processMovedWidgets()
             describedForAppearance = self.describeAppearedWidgets(stateChangeWidgets, shell)
             self.describeStateChanges(stateChanges, describedForAppearance)
-            self.widgetsAppeared = filter(lambda w: self.validAndShowing(w) and self.inDifferentShell(w, shell), self.widgetsAppeared)
-        else:
-            self.widgetsAppeared = []
+            
+        self.widgetsAppeared = filter(lambda w: not w.isDisposed() and self.inDifferentShell(w, shell), self.widgetsAppeared)
         self.parentsResized = set()
         self.widgetsMoved = []
             
@@ -156,7 +240,7 @@ class Describer(storytext.guishared.Describer):
         return not widget.isDisposed() and widget.getShell() == shell
     
     def inDifferentShell(self, widget, shell):
-        return not isinstance(widget, swt.widgets.Shell) and widget.getShell() != shell
+        return not isinstance(widget, Shell) and widget.getShell() != shell
     
     def validAndShowing(self, widget):
         return not widget.isDisposed() and util.isVisible(widget) 
@@ -165,8 +249,9 @@ class Describer(storytext.guishared.Describer):
         return self.validAndShowing(widget) and not self.inDifferentShell(widget, shell)
 
     def describeClipboardChanges(self, display):
-        clipboard = swt.dnd.Clipboard(display)
-        textTransfer = swt.dnd.TextTransfer.getInstance()
+        from org.eclipse.swt.dnd import Clipboard, TextTransfer
+        clipboard = Clipboard(display)
+        textTransfer = TextTransfer.getInstance()
         if self.clipboardText is None:
             # Initially. For some reason it doesn't let us set empty strings here
             # clearContents seemed the way to go, but seems not to work on Windows
@@ -180,10 +265,10 @@ class Describer(storytext.guishared.Describer):
         clipboard.dispose()
         
     def getWindowClasses(self):
-        return swt.widgets.Shell, swt.widgets.Dialog
+        return Shell, Dialog
 
     def getTextEntryClass(self):
-        return swt.widgets.Text
+        return Text
 
     def getWindowString(self):
         return "Shell"
@@ -289,8 +374,15 @@ class Describer(storytext.guishared.Describer):
                                                              subItemMethod=self.getToolItemControls)
     
     def getCoolBarDescription(self, coolbar):
-        return "Cool Bar:\n" + self.getItemBarDescription(coolbar, indent=1,
-                                                          subItemMethod=self.getCoolItemDescriptions)
+        state = self.getCoolBarState(coolbar)
+        self.widgetsWithState[coolbar] = state
+        desc = "Cool Bar"
+        if state:
+            desc += " (" + state + ") "
+        return desc + ":\n" + self.getItemBarDescription(coolbar, indent=1, subItemMethod=self.getCoolItemDescriptions)
+
+    def getCoolBarState(self, coolbar):
+        return colorNameFinder.getNameForWidget(coolbar.getBackground())
 
     def contextMenusEqual(self, menu1, menu2):
         return [ (item.getText(), item.getEnabled()) for item in menu1.getItems() ] == \
@@ -363,6 +455,12 @@ class Describer(storytext.guishared.Describer):
             newImage.dispose()
 
     def getCanvasDescription(self, widget):
+        return self.getAndStoreState(widget)
+    
+    def getCanvasState(self, widget):
+        for canvasDescriberClass in self.canvasDescriberClasses:
+            if canvasDescriberClass.canDescribe(widget):
+                return canvasDescriberClass(widget).getCanvasDescription(self)
         return "Canvas " + self.canvasCounter.getId(widget)
 
     def findStyleList(self, item):
@@ -376,7 +474,7 @@ class Describer(storytext.guishared.Describer):
         style = item.getStyle()
         descs = []
         for tryStyle in styleList:
-            if style & getattr(swt.SWT, tryStyle) != 0:
+            if style & getattr(SWT, tryStyle) != 0:
                 descs.append(tryStyle.lower().replace("_", " ").replace("separator", "---"))
         return descs
 
@@ -398,7 +496,7 @@ class Describer(storytext.guishared.Describer):
         return any((method.getName() == methodName for method in obj.getClass().getDeclaredMethods())) 
         
     def getJfaceTooltip(self, item):
-        for listener in item.getListeners(swt.SWT.MouseHover):
+        for listener in item.getListeners(SWT.MouseHover):
             tooltip = self.getEnclosingInstance(listener)
             if self.hasPrivateMethod(tooltip, "createToolTipContentArea"):
                 return tooltip
@@ -413,7 +511,7 @@ class Describer(storytext.guishared.Describer):
 
     def getControlDecorationListeners(self, item):
         listeners = []
-        for typedListener in item.getListeners(swt.SWT.FocusIn):
+        for typedListener in item.getListeners(SWT.Dispose):
             if hasattr(typedListener, "getEventListener"):
                 focusListener = typedListener.getEventListener()
                 if "ControlDecoration" in focusListener.__class__.__name__:
@@ -428,6 +526,7 @@ class Describer(storytext.guishared.Describer):
                 return field.get(listener)
        
     def getControlDecorationDescription(self, item):
+        texts = []
         for deco in self.getControlDecorations(item):
             if deco:
                 image = deco.getImage()
@@ -437,15 +536,13 @@ class Describer(storytext.guishared.Describer):
                 desc = deco.getDescriptionText()
                 if desc:
                     text += "\n'" + desc + "'"
-                return text
+                texts.append(text)
+        return "\n".join(texts)
             
     def decorationVisible(self, deco):
         if hasattr(deco, "isVisible"): # added in 3.6
             return deco.isVisible()
         else:
-            # Workaround for reflection bug in Jython 2.5.1
-            # args = (None,) if sys.version_info[:3] <= (2, 5, 1) else ()
-            # Jython 2.5.2 doesn't work anyway so we don't include this fix for now
             return util.callPrivateMethod(deco, "shouldShowDecoration")
 
     def isCustomTooltip(self, jfaceTooltip):
@@ -467,10 +564,10 @@ class Describer(storytext.guishared.Describer):
         decoText = self.getControlDecorationDescription(item)
         if decoText:
             elements.append(decoText)
-        if isinstance(item, swt.widgets.Spinner):
+        if isinstance(item, Spinner):
             elements += self.getSpinnerPropertyElements(item)
             
-        jfaceTooltip = enclosingJfaceTooltip if isinstance(item, swt.widgets.Item) else self.getJfaceTooltip(item)
+        jfaceTooltip = enclosingJfaceTooltip if isinstance(item, Item) else self.getJfaceTooltip(item)
         tooltipText = self.getToolTipText(item, jfaceTooltip)
         if tooltipText:
             elements.append("Tooltip '" + tooltipText + "'")
@@ -504,8 +601,8 @@ class Describer(storytext.guishared.Describer):
         return elements
 
     def getLabelState(self, label):
-        if label.getStyle() & swt.SWT.SEPARATOR:
-            if label.getStyle() & swt.SWT.VERTICAL:
+        if label.getStyle() & SWT.SEPARATOR:
+            if label.getStyle() & SWT.VERTICAL:
                 return "-" * 5 + "vertical" + "-" * 5
             else:
                 return "-" * 10
@@ -515,7 +612,7 @@ class Describer(storytext.guishared.Describer):
         for fontData in label.getFont().getFontData():
             fontStyle = fontData.getStyle()
             for fontAttr in [ "BOLD", "ITALIC" ]:
-                if fontStyle & getattr(swt.SWT, fontAttr):
+                if fontStyle & getattr(SWT, fontAttr):
                     elements.append(fontAttr.lower())
         if label.getImage():
             elements.append(self.getImageDescription(label.getImage()))
@@ -542,7 +639,7 @@ class Describer(storytext.guishared.Describer):
 
     def getSashDescription(self, widget):
         orientation = "Horizontal"
-        if widget.getStyle() & swt.SWT.VERTICAL:
+        if widget.getStyle() & SWT.VERTICAL:
             orientation = "Vertical"
         return "-" * 15 + " " + orientation + " sash " + "-" * 15
 
@@ -563,26 +660,30 @@ class Describer(storytext.guishared.Describer):
             return url
         # Ignore non-breaking spaces, they are invisible anyway
         # Webkit returns them in invalid format without the semicolons... handle that too.
-        text = widget.getText().replace(u"\xa0", " ").replace("&nbsp;", " ").replace("&nbsp", " ")
+        text = widget.getText().replace(u"\xa0", " ")
         return BrowserHtmlParser().parse(text)
         
     def getUpdatePrefix(self, widget, oldState, state):
-        if isinstance(widget, (self.getTextEntryClass(), swt.browser.Browser, swt.widgets.Spinner)):
+        if isinstance(widget, (self.getTextEntryClass(), Browser, Spinner)):
             return "\nUpdated " + (util.getTextLabel(widget, useContext=True) or self.getShortWidgetIdentifier(widget) or "Text") +  " Field\n"
-        elif isinstance(widget, (swt.widgets.Combo, swt.custom.CCombo)):
+        elif isinstance(widget, (Combo, CCombo)):
             return "\nUpdated " + util.getTextLabel(widget, useContext=True) + " Combo Box\n"
-        elif util.getTopControl(widget) or isinstance(widget, swt.widgets.Group):
+        elif util.getTopControl(widget) or isinstance(widget, Group):
             return "\n"
-        elif isinstance(widget, swt.widgets.Menu):
+        elif isinstance(widget, Menu):
             parentItem = widget.getParentItem()
             menuRefNr = self.contextMenuCounter.getWidgetNumber(widget)
             menuRefNr = " " + str(menuRefNr) if menuRefNr > 0 else ""
             menuName = parentItem.getText() if parentItem else "Context"
             return "\nUpdated " + menuName + " Menu" + menuRefNr +":\n"
-        elif isinstance(widget, (swt.widgets.Label, swt.custom.CLabel)) and len(state) == 0:
+        elif isinstance(widget, (Label, CLabel)) and len(state) == 0:
             return "\nLabel now empty, previously " + oldState
-        else:
-            return "\nUpdated "
+        elif isinstance(widget, Canvas) and not isinstance(widget, CLabel):
+            for canvasDescriberClass in self.canvasDescriberClasses:
+                if canvasDescriberClass.canDescribe(widget):
+                    return canvasDescriberClass(widget).getUpdatePrefix(oldState, state)
+        
+        return "\nUpdated "
 
     def getShortWidgetIdentifier(self, widget):
         return widget.getData("org.eclipse.swtbot.widget.key")
@@ -631,15 +732,15 @@ class Describer(storytext.guishared.Describer):
         return self.getTextState(widget)
 
     def getDateString(self, widget):
-        if widget.getStyle() & swt.SWT.TIME:
+        if widget.getStyle() & SWT.TIME:
             widgetDate = Date()
             widgetDate.setHours(widget.getHours())
             widgetDate.setMinutes(widget.getMinutes())
             widgetDate.setSeconds(widget.getSeconds())
-            return util.getDateFormat(swt.SWT.TIME).format(widgetDate)
+            return util.getDateFormat(SWT.TIME).format(widgetDate)
         else:
             widgetDate = Date(widget.getYear() - 1900, widget.getMonth(), widget.getDay())
-            return util.getDateFormat(swt.SWT.DATE).format(widgetDate)
+            return util.getDateFormat(SWT.DATE).format(widgetDate)
 
     def getDateTimeState(self, widget):
         elements = [ "DateTime" ] + self.getPropertyElements(widget) + [ "showing " + self.getDateString(widget) ]
@@ -656,7 +757,7 @@ class Describer(storytext.guishared.Describer):
         return text
 
     def getContextMenuReference(self, widget):
-        if "Menu" not in self.excludeClassNames and not isinstance(widget, swt.widgets.MenuItem) and hasattr(widget, "getMenu") and widget.getMenu():
+        if "Menu" not in self.excludeClassNames and not isinstance(widget, MenuItem) and hasattr(widget, "getMenu") and widget.getMenu():
             return "Context Menu " + self.contextMenuCounter.getId(widget.getMenu())
         else:
             return ""
@@ -664,7 +765,7 @@ class Describer(storytext.guishared.Describer):
     def getCustomTooltipReference(self, item, jfaceTooltip):
         if "CustomTooltip" not in self.excludeClassNames and jfaceTooltip and self.isCustomTooltip(jfaceTooltip):
             itemTooltip = self.hasPrivateMethod(jfaceTooltip, "createViewerToolTipContentArea")
-            isItem = isinstance(item, swt.widgets.Item)
+            isItem = isinstance(item, Item)
             if isItem == itemTooltip:
                 return "Custom Tooltip " + self.customTooltipCounter.getId((jfaceTooltip, item))
 
@@ -693,9 +794,9 @@ class Describer(storytext.guishared.Describer):
                                                 columnCount=columnCount,
                                                 enclosingJfaceTooltip=jfaceTooltip)
         sortColumn = widget.getSortColumn()
-        if widget.getSortDirection() == swt.SWT.UP:
+        if widget.getSortDirection() == SWT.UP:
             sortDirection = "(->)"
-        elif widget.getSortDirection() == swt.SWT.DOWN:
+        elif widget.getSortDirection() == SWT.DOWN:
             sortDirection = "(<-)"
         else:
             sortDirection = ""
@@ -757,19 +858,19 @@ class Describer(storytext.guishared.Describer):
     def getVerticalDividePositions(self, children):
         positions = []
         for child in children:
-            if isinstance(child, swt.widgets.Sash) and child.getStyle() & swt.SWT.VERTICAL:
+            if isinstance(child, Sash) and child.getStyle() & SWT.VERTICAL:
                 positions.append(child.getLocation().x)
         return sorted(positions)
 
     def layoutSortsChildren(self, widget):
         layout = widget.getLayout()
-        return layout is not None and isinstance(layout, (swt.layout.GridLayout, swt.layout.FillLayout,
-                                                          swt.layout.RowLayout, swt.custom.StackLayout))
+        return layout is not None and isinstance(layout, (GridLayout, FillLayout,
+                                                          RowLayout, StackLayout))
     
     def _getDescription(self, widget):
         self.widgetsDescribed.add(widget)
         desc = storytext.guishared.Describer._getDescription(self, widget)
-        if desc and isinstance(widget, (swt.widgets.ExpandBar, swt.widgets.Tree, swt.widgets.List, swt.widgets.Table)):
+        if desc and isinstance(widget, (ExpandBar, Tree, List, Table)):
             desc = unicode(desc) + self.formatContextMenuDescriptions()
         return desc
     
@@ -819,9 +920,9 @@ class Describer(storytext.guishared.Describer):
         return text
     
     def makeToolTipEvent(self, widgetOrItem):
-        event = swt.widgets.Event()
-        event.type = swt.SWT.MouseHover
-        if isinstance(widgetOrItem, swt.widgets.Item):
+        event = Event()
+        event.type = SWT.MouseHover
+        if isinstance(widgetOrItem, Item):
             event.widget = widgetOrItem.getParent()
             event.item = widgetOrItem
             bounds = widgetOrItem.getBounds()
@@ -837,8 +938,8 @@ class Describer(storytext.guishared.Describer):
     def getCustomTooltipDescription(self, tooltip, widget):
         event = self.makeToolTipEvent(widget)
         if util.callPrivateMethod(tooltip, "shouldCreateToolTip", [ event ]):
-            shell = swt.widgets.Shell()
-            result = util.callPrivateMethod(tooltip, "createToolTipContentArea", [ event, shell ], [ swt.widgets.Event, swt.widgets.Composite ])
+            shell = Shell()
+            result = util.callPrivateMethod(tooltip, "createToolTipContentArea", [ event, shell ], [ Event, Composite ])
             desc = self.getDescription(result)
             result.dispose()
             shell.dispose()
@@ -854,16 +955,16 @@ class Describer(storytext.guishared.Describer):
             return 1
 
     def usesGrid(self, widget):
-        return isinstance(widget.getLayout(), swt.layout.GridLayout)
+        return isinstance(widget.getLayout(), GridLayout)
 
     def getLayoutColumns(self, widget, childCount, sortedChildren):
         layout = widget.getLayout()
         if hasattr(layout, "numColumns"):
             return layout.numColumns
         elif hasattr(layout, "type"):
-            if layout.type == swt.SWT.HORIZONTAL:
+            if layout.type == SWT.HORIZONTAL:
                 return childCount
-        elif isinstance(layout, swt.layout.FormLayout):
+        elif isinstance(layout, FormLayout):
             currColumns, maxColumns = 1, 1
             for child in sortedChildren:
                 layoutData = child.getLayoutData()
@@ -911,16 +1012,23 @@ class Describer(storytext.guishared.Describer):
 
     def checkWindow(self, window):
         # Don't describe tooltips
-        return (window.getStyle() & swt.SWT.TOOL) == 0
+        return (window.getStyle() & SWT.TOOL) == 0
 
     def splitState(self, state):
         return state if isinstance(state, list) else state.split("\n")
 
     def getStateChangeDescription(self, widget, oldState, state):
-        if isinstance(widget, (swt.widgets.Menu, swt.widgets.ToolBar)):
+        if isinstance(widget, (Menu, ToolBar)):
             old = self.splitState(oldState)
             new = self.splitState(state)
             if len(old) == len(new):
                 return self.getDiffedDescription(widget, old, new)
         return storytext.guishared.Describer.getStateChangeDescription(self, widget, oldState, state)
+    
+    def describeStructure(self, widget, indent=0, **kw):
+        storytext.guishared.Describer.describeStructure(self, widget, indent, **kw)
+        if isinstance(widget, Canvas):
+            for canvasDescriberClass in self.canvasDescriberClasses:
+                if canvasDescriberClass.canDescribe(widget):
+                    canvasDescriberClass(widget).describeCanvasStructure(indent+1)
 
