@@ -1,84 +1,23 @@
 
 from storytext.javaswttoolkit import describer as swtdescriber
 from storytext import guishared, gridformatter
-from util import getInt
-import org.eclipse.draw2d as draw2d
-from org.eclipse import swt
-from java.awt import Color
+import util
 
-class ColorNameFinder:
-    abbrevations = [ ("dark", "+"), ("dull", "#"), ("very", "+"),
-                     ("light", "-"), ("normal", ""), ("bright", "*") ]
-    def __init__(self):
-        self.names = {}
-        # Add java.awt colors  
-        self.addColors(Color, True)
-        # Add swt colors
-        self.addColors(draw2d.ColorConstants)
-    
-    def shortenColorName(self, name):
-        ret = name.lower()
-        for text, repl in self.abbrevations:
-            ret = ret.replace(text, repl)
-        return ret
+from org.eclipse.draw2d import ColorConstants, Figure, FigureUtilities, Graphics, Label, PolylineShape, RectangleFigure
+from org.eclipse.draw2d.geometry import Rectangle
 
-    def addColor(self, name, color, addAwtMark=False):
-        if hasattr(color, "getRed"):
-            newName = name + "&" if addAwtMark else name
-            nameToUse = self.shortenColorName(newName)
-            self.names[self.getRGB(color)] = nameToUse
-            if not addAwtMark:
-                rgb = self.getRGB(draw2d.FigureUtilities.darker(color))
-                if rgb not in self.names:
-                    self.names[rgb] = "D" + nameToUse
-                rgb = self.getRGB(draw2d.FigureUtilities.lighter(color))
-                if rgb not in self.names:
-                    self.names[rgb] = "L" + nameToUse
+from org.eclipse.swt import SWT
+from org.eclipse.swt.graphics import GC
 
-    def addColors(self, cls, addAwtMark=False):
-        for name in sorted(cls.__dict__):
-            if not name.startswith("__"):
-                try:
-                    color = getattr(cls, name)
-                    self.addColor(name, color, addAwtMark)
-                except AttributeError:
-                    pass
+def addColors(cls):
+    swtdescriber.colorNameFinder.addColors(cls, 
+                                           modifiers=[ (FigureUtilities.darker, "D"), (FigureUtilities.lighter, "L")],
+                                           abbreviations=[ ("dark", "+"), ("dull", "#"), ("very", "+"), 
+                                                           ("light", "-"), ("normal", ""), ("bright", "*") ])
 
-    def getRGB(self, color):
-        return color.getRed(), color.getGreen(), color.getBlue()
-
-    def getName(self, color):
-        return self.names.get(self.getRGB(color), "unknown")
-
-colorNameFinder = ColorNameFinder()
-
-
-class Describer(swtdescriber.Describer):
-    def describeStructure(self, widget, indent=0, **kw):
-        swtdescriber.Describer.describeStructure(self, widget, indent, **kw)
-        if self.hasFigureCanvasAPI(widget):
-            self.describeStructure(widget.getContents(), indent+1,
-                                   visibleMethodNameOverride="isVisible", layoutMethodNameOverride="getLayoutManager")
-
-    def hasFigureCanvasAPI(self, widget):
-        # Could just check for FigureCanvas, but sometimes basic Canvases are used with the Figure mechanisms there also
-        # (usually when scrolling not desired to disable mouse-wheel usage)
-        return isinstance(widget, swt.widgets.Canvas) and hasattr(widget, "getContents")
-
-    def getCanvasDescription(self, widget):
-        if hasattr(widget, "getContents"): # FigureCanvas and others sharing its API
-            return self.getAndStoreState(widget)
-        else:
-            return swtdescriber.Describer.getCanvasDescription(self, widget)
-
-    def getCanvasState(self, widget):
-        return FigureCanvasDescriber(widget).getDescription(widget.getContents())
+# Add swt colors
+addColors(ColorConstants)
             
-    def getUpdatePrefix(self, widget, *args):
-        if self.hasFigureCanvasAPI(widget):
-            return "\nUpdated Canvas :\n"
-        else:
-            return swtdescriber.Describer.getUpdatePrefix(self, widget, *args)
 
 class AttrRecorder:
     def __init__(self, name, recorder):
@@ -89,7 +28,7 @@ class AttrRecorder:
         self.recorder.registerCall(self.name, args)
 
         
-class RecorderGraphics(draw2d.Graphics, object):
+class RecorderGraphics(Graphics, object):
     def __init__(self, canvas, font, methodNames):
         self.calls = []
         self.currFont = font
@@ -121,7 +60,7 @@ class RecorderGraphics(draw2d.Graphics, object):
         return 0 # throws NotImplemented by default
     
     def getAntialias(self, *args):
-        return swt.SWT.DEFAULT # throws NotImplemented by default
+        return SWT.DEFAULT # throws NotImplemented by default
     
     def setAntialias(self, *args):
         pass # throws NotImplemented by default
@@ -130,7 +69,7 @@ class RecorderGraphics(draw2d.Graphics, object):
         return self.currFont
     
     def getFontMetrics(self):
-        gc = swt.graphics.GC(self.parentCanvas)
+        gc = GC(self.parentCanvas)
         metrics = gc.getFontMetrics()
         gc.dispose()
         return metrics
@@ -157,17 +96,22 @@ class RecorderGraphics(draw2d.Graphics, object):
         return result
 
 
-class FigureCanvasDescriber(guishared.Describer):
+class CanvasDescriber(util.CanvasDescriber):
     childrenMethodName = "getChildren"
     visibleMethodName = "isVisible"
-    statelessWidgets = [ draw2d.RectangleFigure, draw2d.Label, draw2d.PolylineShape ]
+    statelessWidgets = [ RectangleFigure, Label, PolylineShape ]
     stateWidgets = []
-    ignoreWidgets = [ draw2d.Figure ] # Not interested in anything except what we list
+    ignoreWidgets = [ Figure ] # Not interested in anything except what we list
     ignoreChildren = ()
     pixelTolerance = 2
-    def __init__(self, canvas, *args, **kw):
-        guishared.Describer.__init__(self, *args, **kw)
-        self.canvas = canvas
+    @classmethod
+    def canDescribe(cls, widget):
+        # Could just check for FigureCanvas, but sometimes basic Canvases are used with the Figure mechanisms there also
+        # (usually when scrolling not desired to disable mouse-wheel usage)
+        return hasattr(widget, "getContents")
+        
+    def getCanvasDescription(self, *args):
+        return self.getDescription(self.canvas.getContents())
         
     def getLabelDescription(self, figure):
         return figure.getText()
@@ -179,6 +123,13 @@ class FigureCanvasDescriber(guishared.Describer):
     def paintFigure(self, figure, graphics):
         # So derived classes can reinterpret this if needed, e.g. adding customization to how this is called
         return figure.paintFigure(graphics)
+    
+    def getUpdatePrefix(self, *args):
+        return "\nUpdated Canvas :\n"
+    
+    def describeCanvasStructure(self, indent):
+        self.describeStructure(self.canvas.getContents(), indent,
+                               visibleMethodNameOverride="isVisible", layoutMethodNameOverride="getLayoutManager")
 
     def getRectangleFigureDescription(self, figure):
         font = figure.getFont()
@@ -194,17 +145,17 @@ class FigureCanvasDescriber(guishared.Describer):
             if rectArgs is None: # Colour was set, but no rectangles were made...
                 continue
             
-            rect = draw2d.geometry.Rectangle(*rectArgs)
+            rect = Rectangle(*rectArgs)
             filledRectangles.append(rect)
             colorText = ""
             if colorArgs is not None:
                 rectColor = colorArgs[0]
                 if rectColor != color:
-                    colorText = "(" + colorNameFinder.getName(rectColor) + ")"
+                    colorText = "(" + swtdescriber.colorNameFinder.getName(rectColor) + ")"
                 if rect != bounds and colorText:
                     self.addColouredRectangle(calls, colorText, rect, fontSize)
         calls.sort(cmp=self.compareCalls)
-        colorText = colorNameFinder.getName(color) if self.changedColor(color, figure) else ""
+        colorText = swtdescriber.colorNameFinder.getName(color) if self.changedColor(color, figure) else ""
         if len(graphics.getCallArgs("setAlpha")) > 0:
             colorText = "~" + colorText + "~"
         return self.formatFigure(figure, calls, colorText, filledRectangles)
@@ -226,7 +177,7 @@ class FigureCanvasDescriber(guishared.Describer):
             if rect.contains(x, y) or rect.contains(x, y + fontSize):
                 calls[i] = text + colorText, x, y
                 return
-        calls.append((colorText, getInt(rect.x), getInt(rect.y)))
+        calls.append((colorText, util.getInt(rect.x), util.getInt(rect.y)))
 
     def changedColor(self, color, figure):
         return color != figure.getParent().getBackgroundColor()
@@ -269,7 +220,7 @@ class FigureCanvasDescriber(guishared.Describer):
         return gridformatter.GridFormatter(grid, numColumns)
 
     def usesGrid(self, figure):
-        return isinstance(figure, draw2d.RectangleFigure)
+        return isinstance(figure, RectangleFigure)
 
     def makeTextGrid(self, calls):
         grid = []
@@ -329,7 +280,7 @@ class FigureCanvasDescriber(guishared.Describer):
     def makeCall(self, desc, child):
         loc = child.getLocation()            
         # x and y should be public fields, and are sometimes. In our tests, they are methods, for some unknown reason
-        return desc, getInt(loc.x), getInt(loc.y)
+        return desc, util.getInt(loc.x), util.getInt(loc.y)
             
     def layoutSortsChildren(self, widget):
         return False
