@@ -2,6 +2,7 @@
 """ Generic recorder classes. GUI-specific stuff is in guishared.py """
 
 import os, sys, signal, time, re
+from filepolling import poll_file
 import encodingutils
 from threading import Thread, Timer, Lock
 from definitions import *
@@ -156,9 +157,9 @@ class ReplayScript(object):
                 break
         return comments
 
-    def getCommands(self, args):
+    def getCommands(self, args, matching=[]):
         commands = self.extractAllComments()
-        command = self.getCommand(args)
+        command = self.getCommand(args, matching)
         if command is None:
             return commands
         
@@ -240,7 +241,7 @@ class ShortcutManager:
     
     
 class UseCaseReplayer:
-    def __init__(self, timeout=60):
+    def __init__(self, recorder, timeout=60):
         self.logger = encodingutils.getEncodedLogger("storytext replay log")
         self.scripts = []
         self.shortcutManager = ShortcutManager()
@@ -253,6 +254,9 @@ class UseCaseReplayer:
         self.eventHappenedMessage = ""
         self.appEventTimer = None
         self.appEventTimeout = timeout
+        # For things like comments, and hand-added mutual synch events, which cannot be recorded
+        # Do not use for other purposes! The recorder should be as independent of the replayer as possible
+        self.recorder = recorder
         if os.name == "posix":
             os.setpgrp() # Makes it easier to kill subprocesses
 
@@ -295,11 +299,15 @@ class UseCaseReplayer:
 
     def runScript(self, script, arguments, enableReading):
         if self.shortcutManager.shortcuts:
-            scriptCommand = script.getCommand(arguments, matching=self.shortcutManager.getRegexps())
-            if scriptCommand:
-                newScript, args = self.shortcutManager.findShortcut(scriptCommand)
+            scriptCommands = script.getCommands(arguments, matching=self.shortcutManager.getRegexps())
+            if scriptCommands:
+                while ReplayScript.isComment(scriptCommands[0]):
+                    self.handleComment(scriptCommands[0])
+                    scriptCommands.pop(0)
+
+                newScript, args = self.shortcutManager.findShortcut(scriptCommands[0])
                 self.logger.debug("Found initial shortcut '" + newScript.getShortcutName() + "' with args " + repr(args))
-                if self.addScript(newScript, args, enableReading):
+                if self.addShortcutWithWaits(newScript, args, scriptCommands, enableReading):
                     return
             
         if enableReading and self.processInitialWait(script):
@@ -355,7 +363,7 @@ class UseCaseReplayer:
             self.appEventTimer = None
         self.notifyWaitingCompleted()
 
-    def registerApplicationEvent(self, eventName, timeDelay):
+    def registerApplicationEvent(self, eventName, timeDelay=0.001, **kw):
         origEventName = eventName
         count = 2
         self.appEventLock.acquire()
@@ -421,8 +429,18 @@ class UseCaseReplayer:
             self.eventHappenedMessage = ""
 
     def handleComment(self, comment):
-        pass # Can't do anything here, don't know about the recorder...
+        self.recorder.storeComment(comment)
 
+    def addShortcutWithWaits(self, script, arguments, commands, enableReading=False):
+        self.logger.debug("Found shortcut '" + script.getShortcutName() + "' adding to list of script")
+        if commands[-1].startswith(waitCommandName):
+            self.logger.debug("Adding wait command '" + commands[-1] + "' to the end of a copy of the shortcut file")
+            scriptCopy = copy(script)
+            scriptCopy.addWaitCommand(commands[-1])
+            return self.addScript(scriptCopy, arguments, enableReading)
+        else:
+            return self.addScript(script, arguments, enableReading)
+        
     def runNextCommand(self, **kw):
         if self.timeDelayNextCommand:
             self.logger.debug("Sleeping for " + repr(self.timeDelayNextCommand) + " seconds...")
@@ -439,17 +457,7 @@ class UseCaseReplayer:
             
             script, arguments = self.shortcutManager.findShortcut(command)
             if script:
-                if self.isNonRecursive(script):
-                    self.logger.debug("Found shortcut '" + script.getShortcutName() + "' adding to list of script")
-                    if commands[-1].startswith(waitCommandName):
-                        self.logger.debug("Adding wait command '" + commands[-1] + "' to the end of a copy of the shortcut file")
-                        scriptCopy = copy(script)
-                        scriptCopy.addWaitCommand(commands[-1])
-                        self.addScript(scriptCopy, arguments)
-                    else:
-                        self.addScript(script, arguments)
-                else:
-                    self.writeRecursiveError(script, arguments)
+                self.addShortcutWithWaits(script, arguments, commands)
                 return self.runNextCommand(**kw)
             else:
                 #  Add a delimiter to show that we've replayed something else
@@ -552,10 +560,29 @@ class UseCaseReplayer:
             timer = Timer(subTimerTimeout, self.startTimer, [ timer ])
         self.startTimer(timer)
     
+    def handleMutualSynchWait(self, eventName):
+        fileName = eventName.replace(" ", "_")
+        if os.path.isfile(fileName):
+            os.remove(fileName)
+            self.applicationEventNames.add(eventName)
+            self.logger.debug("Mutual synch completed " + repr(eventName))
+        else:
+            f = open(fileName, "w")
+            try:
+                f.write("Mutual synch file from process " + str(os.getpid()))
+            finally:
+                f.close()
+            self.logger.debug("Mutual synch initiated for " + repr(eventName))
+            poll_file(fileName, eventName, self.registerApplicationEvent)
+        self.recorder.registerApplicationEvent(eventName, category="mutual sych")
+    
     def processWait(self, applicationEventStr):
         eventsToWaitFor = applicationEventStr.split(", ")
         self.describeAppEventsWaiting(eventsToWaitFor)
         self.appEventLock.acquire()
+        for event in eventsToWaitFor:
+            if event.startswith(mutualSynchPrefix):
+                self.handleMutualSynchWait(event)
         allEventsToWaitFor = self.waitingForEvents + eventsToWaitFor
         # Must make sure this is atomic - don't add events one at a time with +=
         self.waitingForEvents = allEventsToWaitFor
